#include "eki_write.h"
#include "../main/iki_write.h"

int main(const int argc, const f_string_t *argv, const f_string_t *envp) {

  iki_write_main_t data = iki_write_main_t_initialize;

  data.program.debug.flag |= iki_write_print_flag_debug_d | iki_write_print_flag_out_d;
  data.program.error.flag |= iki_write_print_flag_error_d | iki_write_print_flag_out_d;
  data.program.output.flag |= iki_write_print_flag_out_d;
  data.program.message.flag |= iki_write_print_flag_message_d | iki_write_print_flag_out_d;
  data.program.warning.flag |= iki_write_print_flag_warning_d | iki_write_print_flag_out_d;
  data.program.error.custom = (void *) &data;
  data.program.debug.custom = (void *) &data;
  data.program.message.custom = (void *) &data;
  data.program.output.custom = (void *) &data;
  data.program.warning.custom = (void *) &data;

  data.callback.load_objects_content_pipe = &iki_write_eki_load_objects_content_pipe;
  data.callback.print_help = &iki_write_eki_print_message_help;
  data.callback.process_objects_content = &iki_write_eki_process_objects_content;

  f_console_parameter_t parameters[] = iki_write_console_parameter_t_initialize;
  data.program.parameters.array = parameters;
  data.program.parameters.used = iki_write_parameter_total_d;
  data.program.environment = envp;

  if (f_pipe_input_exists()) {
    data.program.pipe = fll_program_data_pipe_input_e;
  }

  fll_program_standard_set_up(&data.program);

  f_file_umask_get(&data.program.umask);

  #ifdef _di_thread_support_
    {
      const f_console_arguments_t arguments = macro_f_console_arguments_t_initialize_1(argc, argv, envp);

      iki_write_setting_load(arguments, &data, &iki_write_eki_setting_load);
    }

    iki_write_main(&data);
  #else
    {
      f_thread_id_t id_signal;

      memset(&id_signal, 0, sizeof(f_thread_id_t));

      data.setting.state.status = f_thread_create(0, &id_signal, &iki_write_thread_signal, (void *) &data);

      if (F_status_is_error(data.setting.state.status)) {
        iki_write_print_error(&data.program.error, macro_iki_write_f(f_thread_create));
      }
      else {
        {
          const f_console_arguments_t arguments = macro_f_console_arguments_t_initialize_1(argc, argv, envp);

          iki_write_setting_load(arguments, &data, &iki_write_eki_setting_load);
        }

        if (!iki_write_signal_check(&data)) {
          iki_write_main(&data);
        }

        f_thread_cancel(id_signal);
        f_thread_join(id_signal, 0);
      }
    }
  #endif // _di_thread_support_

  iki_write_main_delete(&data);

  fll_program_standard_set_down(&data.program);

  return F_status_is_error(data.setting.state.status) ? 1 : 0;
}
