#include "eki_write.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_write_print_eki_error_objects_match_content_
  f_status_t iki_write_print_eki_error_objects_match_content(fl_print_t * const print) {

    if (!print) return F_status_set_error(F_output_not);
    if (print->verbosity < f_console_verbosity_error_e) return F_output_not;

    fll_print_format("%[%QThe Objects must have a matching Content.%]%r", print->to, print->set->error, print->prefix, print->set->error, f_string_eol_s);

    return F_okay;
  }
#endif // _di_iki_write_print_eki_error_objects_match_content_

#ifndef _di_iki_write_eki_print_error_objects_not_valid_
  f_status_t iki_write_eki_print_error_objects_not_valid(fl_print_t * const print, const f_string_statics_t objects) {

    if (!print) return F_status_set_error(F_output_not);
    if (print->verbosity < f_console_verbosity_error_e) return F_output_not;

    f_file_stream_lock(print->to);

    fl_print_format("%[%QOne or more of the Objects ", print->to, print->set->error, print->prefix);

    for (f_number_unsigned_t i = 0; i < objects.used; ++i) {

      fl_print_format("'%]", print->to, print->set->error);
      fl_print_format(f_string_format_Q_single_s.string, print->to, print->set->notable, objects.array[i], print->set->notable);
      fl_print_format("%['%Q", print->to, print->set->error);

      if (i + 1 < objects.used) {
        fl_print_format(", ", print->to);
      }
    } // for

    fl_print_format(" is not a valid IKI Object (IKI Vocabulary).%]%r", print->to, print->set->error, f_string_eol_s);

    f_file_stream_unlock(print->to);

    return F_okay;
  }
#endif // _di_iki_write_eki_print_error_objects_not_valid_

#ifndef _di_iki_write_eki_print_message_help_
  void iki_write_eki_print_message_help(fl_print_t * const print) {

    if (!print) return;

    f_file_stream_lock(print->to);

    fll_program_print_help_header(print, iki_write_program_name_long_s, iki_write_program_version_s);

    iki_write_print_message_help(print);

    f_print_dynamic_raw(f_string_eol_s, print->to);

    fll_program_print_help_usage(print, iki_write_program_name_s, f_string_empty_s);

    iki_write_print_message_help_note(print);

    iki_write_eki_print_message_help_note_pipe(print);

    f_file_stream_flush(print->to);
    f_file_stream_unlock(print->to);
  }
#endif // _di_iki_write_eki_print_message_help_

#ifndef _di_iki_write_eki_print_message_help_note_pipe_
  void iki_write_eki_print_message_help_note_pipe(fl_print_t * const print) {

    if (!print) return;
    fl_print_format("%r  When piping main to this program, a single Form-Feed character (\\f) (U+000C) must be used to separate each Object set from the Content.%r", print->to, f_string_eol_s, f_string_eol_s);
    fl_print_format("  Each Object within an Object set is is separated by a single Backspace character (\\b) (U+0008).%r", print->to, f_string_eol_s);
    fl_print_format("  Each Object set must be followed by a Content.%r", print->to, f_string_eol_s);
  }
#endif // _di_iki_write_eki_print_message_help_note_pipe_

#ifdef __cplusplus
} // extern "C"
#endif
