/**
 * FLL - Level 3
 *
 * Project: IKI Write
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 */
#ifndef _eki_write_eki_print_h
#define _eki_write_eki_print_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Print error message about the Objects not having a matching Content.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if setting is NULL.
 */
#ifndef _di_iki_write_print_eki_error_objects_match_content_
  extern f_status_t iki_write_print_eki_error_objects_match_content(fl_print_t * const print);
#endif // _di_iki_write_print_eki_error_objects_match_content_

/**
 * Print error message about an Object not being valid.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 * @param objects
 *   The a strings representing the objects that are not valid.
 *
 * @return
 *   F_okay on success.
 *   F_output_not on success, but no printing is performed.
 *
 *   F_output_not (with error bit) if setting is NULL.
 */
#ifndef _di_iki_write_print_eki_error_objects_not_valid_
  extern f_status_t iki_write_eki_print_error_objects_not_valid(fl_print_t * const print, const f_string_statics_t objects);
#endif // _di_iki_write_print_eki_error_objects_not_valid_

/**
 * Print help for EKI write.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 */
#ifndef _di_iki_write_eki_print_message_help_
  extern void iki_write_eki_print_message_help(fl_print_t * const print);
#endif // _di_iki_write_eki_print_message_help_

/**
 * Print note pipe help for EKI write.
 *
 * @param print
 *   The output structure to print to.
 *
 *   This does not alter print.custom.setting.state.status.
 */
#ifndef _di_iki_write_eki_print_message_help_note_pipe_
  extern void iki_write_eki_print_message_help_note_pipe(fl_print_t * const print);
#endif // _di_iki_write_eki_print_message_help_note_pipe_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _eki_write_eki_print_h
