#include "eki_write.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_write_eki_load_objects_content_pipe_
  void iki_write_eki_load_objects_content_pipe(void * const void_main, const f_range_t range, uint8_t * const pipe_state) {

    if (!void_main) return;

    iki_write_main_t * const main = (iki_write_main_t *) void_main;

    if (!pipe_state || !main->callback.process_objects_content) {
      main->setting.state.status = F_status_set_error(F_parameter);

      iki_write_print_error(&main->program.error, macro_iki_write_f(iki_write_callback_load_objects_content_pipe));

      return;
    }

    if (*pipe_state) {
      if (main->cache.buffer.used) {
        main->setting.state.status = f_string_dynamic_partial_append_nulless(main->cache.buffer, range, &main->cache.content);

        if (F_status_is_error(main->setting.state.status)) {
          iki_write_print_error(&main->program.error, macro_iki_write_f(f_string_dynamic_partial_append_nulless));

          return;
        }
      }

      main->callback.process_objects_content((void *) main, main->cache.objects, main->cache.content);
      if (F_status_is_error(main->setting.state.status)) return;

      fll_print_dynamic_raw(f_string_eol_s, main->program.output.to);

      *pipe_state = 0x0;
      main->cache.objects.used = 0;
      main->cache.content.used = 0;
    }
    else {
      f_range_t range_object = range;
      f_number_unsigned_t total = 0;

      for (; range_object.start <= range_object.stop; ++total) {

        if (iki_write_signal_check(main)) return;

        main->setting.state.status = f_string_dynamic_seek_to(main->cache.buffer, f_string_ascii_space_back_s.string[0], &range_object);

        if (F_status_is_error(main->setting.state.status)) {
          iki_write_print_error(&main->program.error, macro_iki_write_f(f_string_dynamic_seek_to));

          return;
        }

        // Seek past the found colon when the end is not reached.
        if (main->setting.state.status == F_okay) {
          ++range_object.start;
        }
      } // for

      // Designate that no Object is found.
      if (!total) {
        *pipe_state = 0x0;

        return;
      }

      if (main->cache.objects.size < total) {
        f_memory_array_increase_by(total, sizeof(f_string_dynamic_t), (void **) &main->cache.objects.array, &main->cache.objects.used, &main->cache.objects.size);

        if (F_status_is_error(main->setting.state.status)) {
          iki_write_print_error(&main->program.error, macro_iki_write_f(f_memory_array_increase_by));

          return;
        }
      }

      range_object = range;

      f_number_unsigned_t i = 0;
      f_range_t range_next = range;

      for (; i < total; ++i) {

        if (iki_write_signal_check(main)) return;

        main->cache.objects.array[main->cache.objects.used].used = 0;

        main->setting.state.status = f_string_dynamic_seek_to(main->cache.buffer, f_string_ascii_space_back_s.string[0], &range_next);

        if (F_status_is_error(main->setting.state.status)) {
          iki_write_print_error(&main->program.error, macro_iki_write_f(f_string_dynamic_seek_to));

          return;
        }

        range_object.stop = range_next.start - 1;

        main->setting.state.status = f_string_dynamic_partial_append_nulless(main->cache.buffer, range_object, &main->cache.objects.array[main->cache.objects.used]);

        if (F_status_is_error(main->setting.state.status)) {
          iki_write_print_error(&main->program.error, macro_iki_write_f(f_string_dynamic_partial_append_nulless));

          return;
        }

        ++main->cache.objects.used;
        range_object.start = ++range_next.start;
      } // for

      *pipe_state = 0x1;
    }
  }
#endif // _di_iki_write_eki_load_objects_content_pipe_

#ifndef _di_iki_write_eki_process_objects_content_
  void iki_write_eki_process_objects_content(void * const void_main, const f_string_statics_t objects, const f_string_static_t content) {

    if (!void_main) return;

    iki_write_main_t * const main = (iki_write_main_t *) void_main;

    {
      f_number_unsigned_t i = 0;

      for (; i < objects.used; ++i) {
        if (objects.array[i].used) break;
      } // for

      if (!objects.used || i == objects.used) {
        main->setting.state.status = F_status_set_error(F_failure);

        fll_program_print_error_missing_variable_not_zero(&main->program.error, iki_write_object_s);

        return;
      }
    }

    main->cache.iki.used = 0;

    f_iki_eki_write(objects, content, main->setting.quote, &main->cache.iki, &main->setting.state);

    if (F_status_is_error(main->setting.state.status)) {
      if (F_status_set_fine(main->setting.state.status) == F_syntax) {
        iki_write_eki_print_error_objects_not_valid(&main->program.error, objects);
      }
      else {
        iki_write_print_error(&main->program.error, macro_iki_write_f(f_iki_write));
      }

      return;
    }

    f_print_dynamic(main->cache.iki, main->program.output.to);

    main->setting.state.status = F_okay;
  }
#endif // _di_iki_write_eki_process_objects_content_

#ifdef __cplusplus
} // extern "C"
#endif
