/**
 * FLL - Level 3
 *
 * Project: IKI Write
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the common data structures.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_write_iki_common_h
#define _iki_write_iki_common_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * The IKI write specific strings.
 */
#ifndef _di_iki_write_iki_s_
  #define IKI_WRITE_program_name_s      "iki_write"
  #define IKI_WRITE_program_name_long_s "IKI Write"

  #define IKI_WRITE_program_name_s_length      9
  #define IKI_WRITE_program_name_long_s_length 9

  extern const f_string_static_t iki_write_program_name_s;
  extern const f_string_static_t iki_write_program_name_long_s;
#endif // _di_iki_write_iki_s_

/**
 * Perform the program setting load process for IKI.
 *
 * This prints error messages as appropriate.
 *
 * If either main or setting is NULL, then this immediately returns without doing anything.
 *
 * @param arguments
 *   The parameters passed to the process (often referred to as command line arguments).
 * @param main
 *   The program and settings data.
 *
 *   This alters main.setting.state.status:
 *     F_okay on success.
 *
 *     Errors (with error bit) from: f_console_parameter_process().
 *     Errors (with error bit) from: f_file_stream_open().
 *     Errors (with error bit) from: fll_program_parameter_process_context().
 *     Errors (with error bit) from: fll_program_parameter_process_verbosity_standard().
 *
 * @see f_console_parameter_process()
 * @see f_file_stream_open()
 * @see fll_program_parameter_process_context()
 * @see fll_program_parameter_process_verbosity_standard()
 */
#ifndef _di_iki_write_iki_setting_load_
  extern void iki_write_iki_setting_load(const f_console_arguments_t arguments, iki_write_main_t * const main);
#endif // _di_iki_write_iki_setting_load_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_write_iki_common_h
