#include "iki_write.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_write_iki_load_objects_content_pipe_
  void iki_write_iki_load_objects_content_pipe(void * const void_main, const f_range_t range, uint8_t * const pipe_state) {

    if (!void_main) return;

    iki_write_main_t * const main = (iki_write_main_t *) void_main;

    if (!pipe_state || !main->callback.process_objects_content) {
      main->setting.state.status = F_status_set_error(F_parameter);

      iki_write_print_error(&main->program.error, macro_iki_write_f(iki_write_callback_load_objects_content_pipe));

      return;
    }

    if (*pipe_state) {
      main->cache.content.used = 0;

      if (main->cache.buffer.used) {
        main->setting.state.status = f_string_dynamic_partial_append_nulless(main->cache.buffer, range, &main->cache.content);

        if (F_status_is_error(main->setting.state.status)) {
          iki_write_print_error(&main->program.error, macro_iki_write_f(f_string_dynamic_partial_append_nulless));

          return;
        }
      }

      main->callback.process_objects_content((void *) main, main->cache.objects, main->cache.content);
      if (F_status_is_error(main->setting.state.status)) return;

      fll_print_dynamic_raw(f_string_eol_s, main->program.output.to);

      *pipe_state = 0x0;
      main->cache.objects.used = 0;
    }
    else {
      main->cache.objects.used = 0;

      f_memory_array_increase(main->setting.state.step_small, sizeof(f_string_dynamic_t), (void **) &main->cache.objects.array, &main->cache.objects.used, &main->cache.objects.size);

      if (F_status_is_error(main->setting.state.status)) {
        iki_write_print_error(&main->program.error, macro_iki_write_f(f_memory_array_increase));

        return;
      }

      main->cache.objects.array[main->cache.objects.used].used = 0;

      main->setting.state.status = f_string_dynamic_partial_append_nulless(main->cache.buffer, range, &main->cache.objects.array[0]);

      if (F_status_is_error(main->setting.state.status)) {
        iki_write_print_error(&main->program.error, macro_iki_write_f(f_string_dynamic_partial_append_nulless));

        return;
      }

      *pipe_state = 0x1;
      ++main->cache.objects.used;
    }
  }
#endif // _di_iki_write_iki_load_objects_content_pipe_

#ifndef _di_iki_write_iki_process_objects_content_
  void iki_write_iki_process_objects_content(void * const void_main, const f_string_statics_t objects, const f_string_static_t content) {

    if (!void_main) return;

    iki_write_main_t * const main = (iki_write_main_t *) void_main;

    // Only the first index is used because IKI only supports one Object (Vocabulary).
    if (!objects.used || !objects.array[0].used) {
      main->setting.state.status = F_status_set_error(F_failure);

      fll_program_print_error_missing_variable_not_zero(&main->program.error, iki_write_object_s);

      return;
    }

    main->cache.iki.used = 0;

    f_iki_write(objects.array[0], content, main->setting.quote, &main->cache.iki, &main->setting.state);

    if (F_status_is_error(main->setting.state.status)) {
      if (F_status_set_fine(main->setting.state.status) == F_syntax) {
        iki_write_iki_print_error_object_not_valid(&main->program.error, objects.array[0]);
      }
      else {
        iki_write_print_error(&main->program.error, macro_iki_write_f(f_iki_write));
      }

      return;
    }

    f_print_dynamic(main->cache.iki, main->program.output.to);

    main->setting.state.status = F_okay;
  }
#endif // _di_iki_write_iki_process_objects_content_

#ifdef __cplusplus
} // extern "C"
#endif
