/**
 * FLL - Level 3
 *
 * Project: IKI Write
 * API Version: 0.7
 * Licenses: lgpl-2.1-or-later
 *
 * Provides the process functionality.
 *
 * This is auto-included and should not need to be explicitly included.
 */
#ifndef _iki_write_iki_process_h
#define _iki_write_iki_process_h

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Load the Objects and Content from the given pipe information.
 *
 * This function prints errors.
 *
 * @param main
 *   The program and settings data.
 *
 *   Must not be NULL.
 *
 *   This alters main.setting.state.status:
 *     F_okay on success.
 *
 *     Errors (with error bit) from: f_iki_write().
 *
 *     F_failure (with error bit) for any other failure.
 * @param range
 *   The range within the buffer representing the objects.
 * @param pipe_state
 *   Designate that the Object part of the pipe is complete.
 *
 *   State Bits:
 *     - 0x0: No Object has been found.
 *     - 0x1: Object is found and processed.
 *
 * @see f_iki_write()
 */
#ifndef _di_iki_write_iki_load_objects_content_pipe_
  extern void iki_write_iki_load_objects_content_pipe(void * const main, const f_range_t range, uint8_t * const pipe_state);
#endif // _di_iki_write_iki_load_objects_content_pipe_

/**
 * Process a given object and content, printing it.
 *
 * This function prints errors.
 *
 * @param main
 *   The program and settings data.
 *
 *   Must not be NULL.
 *
 *   This alters main.setting.state.status:
 *     F_okay on success.
 *
 *     Errors (with error bit) from: f_iki_write().
 *
 *     F_failure (with error bit) for any other failure.
 * @param objects
 *   The objects to process and print.
 * @param content
 *   The content to process and print.
 *
 * @see f_iki_write()
 */
#ifndef _di_iki_write_iki_process_objects_content_
  extern void iki_write_iki_process_objects_content(void * const main, const f_string_statics_t objects, const f_string_static_t content);
#endif // _di_iki_write_iki_process_objects_content_

#ifdef __cplusplus
} // extern "C"
#endif

#endif // _iki_write_iki_process_h
