#include "iki_write.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_write_setting_load_
  void iki_write_setting_load(const f_console_arguments_t arguments, iki_write_main_t * const main, void (*callback)(const f_console_arguments_t arguments, iki_write_main_t * const main)) {

    if (!main) return;

    main->program.output.to.id = F_type_descriptor_output_d;
    main->program.output.to.stream = F_type_output_d;
    main->program.output.to.flag = F_file_flag_create_d | F_file_flag_write_only_d | F_file_flag_append_d;

    f_console_parameters_t * const parameters = &main->program.parameters;

    main->setting.state.step_small = iki_write_allocation_console_d;

    f_console_parameter_process(arguments, parameters, &main->setting.state, 0);

    main->setting.state.step_small = iki_write_allocation_small_d;

    if (F_status_is_error(main->setting.state.status)) {
      iki_write_print_error(&main->program.error, macro_iki_write_f(f_console_parameter_process));

      return;
    }

    main->setting.state.status = fll_program_parameter_process_context_standard(F_true, &main->program);

    if (F_status_is_error(main->setting.state.status)) {
      iki_write_print_error(&main->program.error, macro_iki_write_f(fll_program_parameter_process_context_standard));

      return;
    }

    main->setting.state.status = fll_program_parameter_process_verbosity_standard(F_true, &main->program);

    if (F_status_is_error(main->setting.state.status)) {
      iki_write_print_error(&main->program.error, macro_iki_write_f(fll_program_parameter_process_verbosity_standard));

      return;
    }

    if (callback) {
      callback(arguments, main);
      if (F_status_is_error(main->setting.state.status)) return;

      if (main->setting.state.status == F_done) {
        main->setting.state.status = F_okay;

        return;
      }
    }

    if (parameters->array[f_console_standard_parameter_help_e].result & f_console_result_found_e) {
      main->setting.flag |= iki_write_main_flag_help_d;

      return;
    }

    if (parameters->array[f_console_standard_parameter_version_e].result & f_console_result_found_e) {
      main->setting.flag |= iki_write_main_flag_version_d;

      return;
    }

    if (parameters->array[f_console_standard_parameter_copyright_e].result & f_console_result_found_e) {
      main->setting.flag |= iki_write_main_flag_copyright_d;

      return;
    }

    if (main->program.pipe & fll_program_data_pipe_input_e) {
      main->setting.flag |= iki_write_main_flag_pipe_d;
    }
    else {
      main->setting.flag &= ~iki_write_main_flag_pipe_d;
    }

    if (parameters->array[iki_write_parameter_file_e].result & f_console_result_value_e && parameters->array[iki_write_parameter_file_e].values.used) {
      if (parameters->array[iki_write_parameter_file_e].values.used > 1) {
        main->setting.state.status = F_status_set_error(F_parameter);

        fll_program_print_error_parameter_must_specify_once(&main->program.error, f_console_symbol_long_normal_s, iki_write_long_file_s);

        return;
      }

      const f_number_unsigned_t index = parameters->array[iki_write_parameter_file_e].values.array[0];

      main->program.output.to.id = -1;
      main->program.output.to.stream = 0;

      main->setting.state.status = f_file_stream_open(parameters->arguments.array[index], f_string_empty_s, &main->program.output.to);

      if (F_status_is_error(main->setting.state.status)) {
        fll_error_file_print(&main->program.error, F_status_set_fine(main->setting.state.status), macro_iki_write_f(f_file_stream_open), fll_error_file_flag_fallback_e, parameters->arguments.array[index], f_file_operation_open_s, fll_error_file_type_file_e);

        return;
      }

      main->setting.flag |= iki_write_main_flag_file_to_d;
    }
    else if (parameters->array[iki_write_parameter_file_e].result & f_console_result_found_e) {
      main->setting.state.status = F_status_set_error(F_parameter);

      fll_program_print_error_parameter_missing_value(&main->program.error, f_console_symbol_long_normal_s, iki_write_long_file_s);

      return;
    }

    if (!(main->program.pipe & fll_program_data_pipe_input_e) && !(main->setting.flag & (iki_write_main_flag_content_d | iki_write_parameter_object_e))) {
      main->setting.state.status = F_status_set_error(F_parameter);

      iki_write_print_error_main_missing(&main->program.error);

      return;
    }

    if (parameters->array[iki_write_parameter_wrap_e].result & f_console_result_found_e) {
      main->setting.flag |= iki_write_main_flag_wrap_d;
      main->setting.state.flag |= f_iki_state_flag_force_wrap_e;
    }
    else {
      main->setting.state.flag &= ~f_iki_state_flag_force_wrap_e;
    }

    main->setting.quote = f_iki_syntax_quote_double_s;

    if (parameters->array[iki_write_parameter_double_e].result & f_console_result_found_e) {
      if (parameters->array[iki_write_parameter_single_e].result & f_console_result_found_e) {
        if (parameters->array[iki_write_parameter_double_e].location < parameters->array[iki_write_parameter_single_e].location) {
          main->setting.quote = f_iki_syntax_quote_single_s;

          if (parameters->array[iki_write_parameter_grave_e].result & f_console_result_found_e) {
            if (parameters->array[iki_write_parameter_single_e].location < parameters->array[iki_write_parameter_grave_e].location) {
              main->setting.quote = f_iki_syntax_quote_grave_s;
            }
          }
        }
        else if (parameters->array[iki_write_parameter_grave_e].result & f_console_result_found_e) {
          if (parameters->array[iki_write_parameter_double_e].location < parameters->array[iki_write_parameter_grave_e].location) {
            main->setting.quote = f_iki_syntax_quote_grave_s;
          }
        }
      }
      else if (parameters->array[iki_write_parameter_grave_e].result & f_console_result_found_e) {
        if (parameters->array[iki_write_parameter_double_e].location < parameters->array[iki_write_parameter_grave_e].location) {
          main->setting.quote = f_iki_syntax_quote_grave_s;
        }
      }
    }
    else if (parameters->array[iki_write_parameter_single_e].result & f_console_result_found_e) {
      main->setting.quote = f_iki_syntax_quote_single_s;

      if (parameters->array[iki_write_parameter_grave_e].result & f_console_result_found_e) {
        if (parameters->array[iki_write_parameter_single_e].location < parameters->array[iki_write_parameter_grave_e].location) {
          main->setting.quote = f_iki_syntax_quote_grave_s;
        }
      }
    }
    else if (parameters->array[iki_write_parameter_grave_e].result & f_console_result_found_e) {
      main->setting.quote = f_iki_syntax_quote_grave_s;
    }
  }
#endif // _di_iki_write_setting_load_

#ifdef __cplusplus
} // extern "C"
#endif
