#include "iki_write.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _di_iki_write_main_
  void iki_write_main(iki_write_main_t * const main) {

    if (!main || F_status_is_error(main->setting.state.status)) return;

    main->setting.state.status = F_okay;

    if (main->setting.flag & iki_write_main_flag_version_copyright_help_d) {
      if (main->setting.flag & iki_write_main_flag_help_d) {
        if (main->callback.print_help) {
          main->callback.print_help(&main->program.output);
        }
      }
      else if (main->setting.flag & iki_write_main_flag_version_d) {
        fll_program_print_version(&main->program.message, iki_write_program_version_s);
      }
      else if (main->setting.flag & iki_write_main_flag_copyright_d) {
        fll_program_print_copyright(&main->program.message, fll_program_copyright_year_author_s);
      }

      return;
    }

    if (main->setting.flag & iki_write_main_flag_pipe_d) {
      uint8_t pipe_state = 0x0;
      f_number_unsigned_t previous = 0;
      f_range_t range = f_range_t_initialize;
      f_status_t status = F_okay;
      f_file_t pipe = f_file_t_initialize;

      pipe.id = F_type_descriptor_input_d;
      pipe.size_read = 1;

      main->cache.buffer.used = 0;
      main->cache.content.used = 0;
      main->cache.objects.used = 0;

      range.start = 0;

      do {
        if (iki_write_signal_check(main)) return;

        if (status != F_okay_eof) {
          status = f_file_read(pipe, &main->cache.buffer);

          if (F_status_is_error(status)) {
            main->setting.state.status = F_status_set_error(F_pipe);

            iki_write_print_error_file(&main->program.error, macro_iki_write_f(f_file_read), f_string_ascii_minus_s, f_file_operation_read_s, fll_error_file_type_pipe_e);

            break;
          }

          if (!main->cache.buffer.used) {
            main->setting.state.status = F_status_set_error(F_parameter);

            fll_program_print_error_pipe_missing_content(&main->program.error);

            break;
          }

          range.stop = main->cache.buffer.used - 1;
        }

        previous = range.start;
        main->setting.state.status = f_string_dynamic_seek_to(main->cache.buffer, f_string_ascii_feed_form_s.string[0], &range);

        if (main->setting.state.status == F_data_not_stop) {
          main->setting.state.status = F_status_set_error(F_parameter);
        }

        if (F_status_is_error(main->setting.state.status)) {
          iki_write_print_error(&main->program.error, macro_iki_write_f(f_string_dynamic_seek_to));

          break;
        }

        if (pipe_state && previous == range.start) {
          main->setting.state.status = F_status_set_error(F_parameter);

          fll_program_print_error_pipe_invalid_form_feed(&main->program.error);

          break;
        }

        range.stop = range.start - 1;
        range.start = previous;

        if (main->callback.load_objects_content_pipe) {
          main->callback.load_objects_content_pipe((void *) main, range, &pipe_state);
          if (F_status_is_error(main->setting.state.status)) break;
        }

        // Restore the range, positioned after the new line.
        range.start = range.stop + 2;
        range.stop = main->cache.buffer.used - 1;

        // Only clear the buffer and reset the start when the entire buffer has been processed.
        if (range.start > range.stop) {
          range.start = 0;
          main->cache.buffer.used = 0;
        }

      } while (status != F_okay_eof || main->cache.buffer.used || pipe_state);

      if (F_status_is_error_not(main->setting.state.status) && pipe_state) {
        main->setting.state.status = F_status_set_error(F_parameter);

        fll_program_print_error_pipe_object_without_content(&main->program.error);
      }
    }

    if (F_status_is_error_not(main->setting.state.status) && F_status_set_fine(main->setting.state.status) != F_interrupt) {
      for (f_number_unsigned_t i = 0; i < main->setting.objectss.used; ++i) {

        if (iki_write_signal_check(main)) return;

        main->callback.process_objects_content((void *) main, main->setting.objectss.array[i], main->setting.contents.array[i]);
        if (F_status_is_error(main->setting.state.status)) break;

        fll_print_dynamic_raw(f_string_eol_s, main->program.output.to);
      } // for
    }
  }
#endif // _di_iki_write_main_

#ifdef __cplusplus
} // extern "C"
#endif
